from scrapy.crawler import CrawlerProcess
from scrapy import log


def run_spider(spider, settings, loglevel='INFO'):
    """
    Run a spider with given settings
    """
    crawler = CrawlerProcess(settings)
    crawler.install()
    crawler.configure()
    crawler.crawl(spider)
    log.start(loglevel=loglevel)
    crawler.start()



from scrapy.xlib.pydispatch import dispatcher
from scrapy.crawler import CrawlerProcess
from scrapy import signals, log

from scrapy.commands.check import Command as CheckCommand
from scrapy.contracts import ContractsManager
from scrapy.utils.misc import load_object
from scrapy.utils.conf import build_component_list

from nose.plugins.xunit import Xunit
from nose.config import Config


class AttributeDict(dict):
    __getattr__ = dict.__getitem__


def run_tests(spider, output_file, settings):
    """
    Helper for running test contractors for a spider and output an
    XUnit file (for CI)

    For using offline input the HTTP cache is enabled
    """

    settings.overrides.update({
        "HTTPCACHE_ENABLED": True,
        "HTTPCACHE_EXPIRATION_SECS": 0,
    })

    crawler = CrawlerProcess(settings)

    contracts = build_component_list(
        crawler.settings['SPIDER_CONTRACTS_BASE'],
        crawler.settings['SPIDER_CONTRACTS'],
    )

    xunit = Xunit()
    xunit.enabled = True
    xunit.configure(AttributeDict(xunit_file=output_file), Config())
    xunit.stopTest = lambda *x: None

    check = CheckCommand()
    check.set_crawler(crawler)
    check.settings = settings
    check.conman = ContractsManager([load_object(c) for c in contracts])
    check.results = xunit
    # this are specially crafted requests that run tests as callbacks
    requests = check.get_requests(spider)

    crawler.install()
    crawler.configure()
    crawler.crawl(spider, requests)
    log.start(loglevel='DEBUG')

    # report is called when the crawler finishes, it creates the XUnit file
    report = lambda: check.results.report(check.results.error_report_file)
    dispatcher.connect(report, signals.engine_stopped)

    crawler.start()



try:
    from setuptools import setup
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()
    from setuptools import setup

install_requires = [
    'Scrapy>=0.16.4',
    'nose==1.2.1',
]

long_description = open('README.rst').read()

setup(
    name='scrapyrwiki',
    version="0.1",
    description='A collection of helpers for running Scrapy in ScraperWiki',
    url='http://github.com/SpazioDati/scrapyrwiki',
    author='SpazioDati',
    author_email='hello@spaziodati.eu',
    packages=['scrapyrwiki'],
    zip_safe=True,
    license='BSD',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Utilities'
    ],
    long_description=long_description,
    install_requires=install_requires,
)



from collections import defaultdict

from scrapy.xlib.pydispatch import dispatcher
from scrapy import signals


def gen_table_name(item_type):
    table_name = item_type.lower()
    if table_name.endswith("item") and len(table_name) > 4:
        table_name = table_name[:-4]
    return table_name


class ScraperWikiPipeline(object):
    """
    A pipeline for saving to the Scraperwiki datastore

    If the scraper returns different kind of items they are stored in
    different tables
    """
    def __init__(self):
        self.buff = 20
        self.data = defaultdict(list)
        self.counter = 0
        dispatcher.connect(self.spider_closed, signals.spider_closed)

    def process_item(self, item, spider):
        item_type = item.__class__.__name__
        self.data[item_type].append(dict(item))
        if len(self.data[item_type]) >= self.buff:
            self.write_data(spider, item_type)
        return item

    def spider_closed(self, spider):
        for item_type in self.data:
            if self.data[item_type]:
                self.write_data(spider, item_type)

    def write_data(self, spider, item_type):
        import scraperwiki

        table_name = gen_table_name(item_type)
        unique_keys = spider.settings.get(
            'SW_UNIQUE_KEYS', {item_type: ['id']}
        )
        scraperwiki.sqlite.save(
            table_name=table_name,
            unique_keys=unique_keys[item_type],
            data=self.data[item_type]
        )
        self.data[item_type] = []
